import logging
import os

import sleekxmpp


class NtfySendMsgBot(sleekxmpp.ClientXMPP):
    """
    Modified the commented sleekxmpp example:
    http://sleekxmpp.com/getting_started/sendlogout.html

    NOTE: supplying mtype='chat' was required for
          Google Hangouts to work
    """

    def __init__(self, jid, password, recipient, title, message, mtype=None):
        super(NtfySendMsgBot, self).__init__(jid, password)

        self.recipient = recipient
        self.title = title
        self.msg = message
        self.mtype = mtype

        self.add_event_handler("session_start", self.start)

    def start(self, event):

        self.send_presence()
        self.get_roster()
        msg_args = {
            'mto': self.recipient,
            'msubject': self.title,
            'mbody': self.msg
        }
        if self.mtype:
            msg_args['mtype'] = self.mtype

        self.send_message(**msg_args)

        self.disconnect(wait=True)


def notify(title,
           message,
           jid,
           password,
           recipient,
           hostname=None,
           port=5222,
           path_to_certs=None,
           mtype=None,
           retcode=None):
    """
    Optional parameters
        * ``hostname`` (if not from jid)
        * ``port``
        * ``path_to_certs``
        * ``mtype`` ('chat' required for Google Hangouts)

    To verify the SSL certificates offered by a server:
    path_to_certs = "path/to/ca/cert"

    Without dnspython library installed, you will need
    to specify the server hostname if it doesn't match the jid.

    For example, to use Google Talk you would need to use:
    hostname = 'talk.google.com'

    Specify port if other than 5222.
    NOTE: Ignored without specified hostname
    """

    xmpp_bot = NtfySendMsgBot(jid, password, recipient, title, message, mtype)

    # NOTE: Below plugins weren't needed for Google Hangouts
    # but may be useful (from original sleekxmpp example)
    # xmpp_bot.register_plugin('xep_0030') # Service Discovery
    # xmpp_bot.register_plugin('xep_0199') # XMPP Ping

    if path_to_certs and os.path.isdir(path_to_certs):
        xmpp_bot.ca_certs = path_to_certs

    # Connect to the XMPP server and start processing XMPP stanzas.
    if xmpp_bot.connect(*([(hostname, int(port)) if hostname else []])):
        xmpp_bot.process(block=True)
    else:
        logging.getLogger(__name__).error('Unable to connect', exc_info=True)



from matrix_client.client import MatrixClient

def notify(title, message, url, roomId, userId=None, token=None, password=None, retcode=None):

    client = MatrixClient(url)
    if password is not None:
        if userId is None:
            raise Exception("Must supply 'userid' when using 'password'")
        client.login(userId, password, sync=False)
    elif token is not None:
        client.api.token = token
    else:
        raise Exception("Must supply 'token' or 'password'")
    msg_plain = "**{}** {}".format(title, message)
    msg_html = "<b>{}</b> {}".format(title, message)
    room = client.join_room(roomId)
    room.send_html(msg_html, msg_plain)



from importlib import import_module
from sys import platform


class DefaultNotifierError(Exception):
    def __init__(self, exception, module):
        self.exception = exception
        self.module = module


def notify(title, message, **kwargs):
    """
    This backend automatically selects the correct desktop notification backend
    for your operating system.
    """
    for os in ['linux', 'win32', 'darwin']:
        if platform.startswith(os):
            module = import_module('ntfy.backends.{}'.format(os))
            try:
                module.notify(title=title, message=message, **kwargs)
            except Exception as e:
                raise DefaultNotifierError(e, module)
            break



from os import environ, path

from ..data import icon


def notify(title,
           message,
           icon=icon.png,
           urgency=None,
           transient=None,
           soundfile=None,
           timeout=-1,
           retcode=0):
    try:
        import dbus
    except ImportError:
        import sys
        import logging
        logger = logging.getLogger(__name__)
        if sys.platform.startswith('linux') and hasattr(sys, 'real_prefix'):
            logger.error(
                'Using ntfy for Linux desktop notifications with '
                'virtualenv requires creating your virtualenv with the '
                '--system-site-packages option')
            return
        elif not environ.get('DISPLAY'):
            logger.warn('DISPLAY not set')
            return
        else:
            raise

    bus = dbus.SessionBus()
    dbus_obj = bus.get_object('org.freedesktop.Notifications',
                              '/org/freedesktop/Notifications')
    dbus_iface = dbus.Interface(
        dbus_obj, dbus_interface='org.freedesktop.Notifications')

    hints = {}

    # Override the retcode if the urgency
    if urgency == 'low':
        hints = {'urgency': dbus.Byte(0)}
    elif urgency == 'normal':
        hints = {'urgency': dbus.Byte(1)}
    elif urgency == 'critical':
        hints = {'urgency': dbus.Byte(2)}
    # Fallback to the return code to determine the urgency flag.
    elif retcode:
        hints = {'urgency': dbus.Byte(2)}

    if str(transient).lower() == 'true':
        hints.update({'transient': dbus.Byte(1)})
    elif str(transient).lower() == 'false':
        hints.update({'transient': dbus.Byte(0)})
    elif transient is not None:
        logger.warn(
            'Unexpected value for the "transient" option. Expected values (true, false).'
        )

    if soundfile:
        hints.update({'sound-file': soundfile})

    message = message.replace('&', '&amp;')
    dbus_iface.Notify('ntfy', 0, "" if not icon else path.abspath(icon), title, 
                      message, [], hints, int(timeout))



import shlex
from os import environ, ttyname
from subprocess import PIPE, Popen, check_output, CalledProcessError
from sys import platform, stdout


def linux_window_is_focused():
    xprop_cmd = shlex.split('xprop -root _NET_ACTIVE_WINDOW')
    try:
        xprop_window_id = int(check_output(xprop_cmd, stdout=PIPE, stderr=PIPE).split()[-1], 16)
    except CalledProcessError:
        return False
    except ValueError:
        return False
    except OSError as e:
        if 'No such file' in e.strerror:
            return False
        else:
            raise
    env_window_id = int(environ.get('WINDOWID', '0'))
    return env_window_id == xprop_window_id


def osascript_tell(app, script):
    p = Popen(['osascript'], stdin=PIPE, stdout=PIPE)
    stdout, stderr = p.communicate(
        ('tell application "{}"\n{}\nend tell'.format(app, script)
         .encode('utf-8')))
    return stdout.decode('utf-8').rstrip('\n')


def darwin_iterm2_shell_is_focused():
    focused_tty = osascript_tell(
        'iTerm',
        'tty of current session of current window',
    )
    return focused_tty == ttyname(stdout.fileno())


def darwin_terminal_shell_is_focused():
    focused_tty = osascript_tell(
        'Terminal',
        'tty of (first tab of (first window whose frontmost is true) '
        'whose selected is true)',
    )
    return focused_tty == ttyname(stdout.fileno())


def darwin_app_shell_is_focused():
    current_appid = {
        'iTerm.app': 'iTerm2',
        'Apple_Terminal': 'Terminal',
    }.get(environ.get('TERM_PROGRAM'))
    focused_appid = osascript_tell(
        'System Events',
        'name of first application process whose frontmost is true',
    )
    if current_appid == focused_appid:
        return {
            'Terminal': darwin_terminal_shell_is_focused,
            'iTerm2': darwin_iterm2_shell_is_focused,
        }[current_appid]()


def is_focused():
    if platform.startswith('linux') and environ.get('DISPLAY'):
        return linux_window_is_focused()
    elif platform == 'darwin':
        return darwin_app_shell_is_focused()
    else:
        return False



import requests

from ..config import USER_AGENT

NTFY_API_KEY = '7fb59b2bedc4df26afa306d5dc54495b6394295a'
API_URL = 'https://api.prowlapp.com/publicapi/add'
MIN_PRIORITY = -2
MAX_PRIORITY = 2


def notify(title,
           message,
           api_key=NTFY_API_KEY,
           provider_key=None,
           priority=0,
           url=None,
           retcode=None):
    """
    Optional parameters:
        * ``api_key`` - use your own application token
        * ``provider_key`` - if you are whitelisted
        * ``priority``
        * ``url``
    """

    data = {
        'apikey': api_key,
        'application': 'ntfy',
        'event': title,
        'description': message,
    }

    if MIN_PRIORITY <= priority <= MAX_PRIORITY:
        data['priority'] = priority
    else:
        raise ValueError('priority must be an integer from {:d} to {:d}'
                         .format(MIN_PRIORITY, MAX_PRIORITY))

    if url is not None:
        data['url'] = url

    if provider_key is not None:
        data['providerkey'] = provider_key

    resp = requests.post(
        API_URL, data=data, headers={
            'User-Agent': USER_AGENT,
        })

    resp.raise_for_status()



import requests

from ..config import USER_AGENT


def notify(title, message, key, event=None, retcode=None):
    """
    Required paramter:
        * ``key`` - The Simplepush identification key, created by
        installing the Android App (https://simplepush.io)

    Optional parameters:
        * ``event`` - use custom ringtones and vibration patterns
    """

    data = {
        'title': title if len(title) <= 20 else title[:19] + u'\u2026',
        'msg': message,
        'key': key
    }

    if event:
        data['event'] = event

    headers = {'User-Agent': USER_AGENT}

    endpoint = "https://api.simplepush.io"

    resp = requests.post(endpoint + '/send', data=data, headers=headers)

    resp.raise_for_status()



import logging
from getpass import getuser
from os import getcwd, path, name
from socket import gethostname
from importlib import import_module
from inspect import getargspec
from .backends.default import DefaultNotifierError

__version__ = '2.7.0'

_user_home = path.expanduser('~')
_cwd = getcwd()
if name != 'nt' and _cwd.startswith(_user_home):
    default_title = '{}@{}:{}'.format(
        getuser(), gethostname(), path.join('~', _cwd[len(_user_home) + 1:]))
else:
    default_title = '{}@{}:{}'.format(getuser(), gethostname(), _cwd)


def notify(message, title, config=None, **kwargs):
    from .config import load_config

    if config is None:
        config = load_config()

    ret = 0
    retcode = kwargs.pop('retcode', None)

    for backend in config.get('backends', ['default']):
        backend_config = config.get(backend, {})
        backend_config.update(kwargs)
        if 'backend' in backend_config:
            backend = backend_config.pop('backend')

        if title is None:
            title = backend_config.pop('title',
                                       config.get('title', default_title))
        elif 'title' in backend_config:
            del backend_config['title']

        try:
            notifier = import_module('ntfy.backends.{}'.format(backend))
        except ImportError:
            try:
                notifier = import_module(backend)
            except ImportError:
                logging.getLogger(__name__).error(
                    'Invalid backend {}'.format(backend))
                ret = 1
                continue

        try:
            notify_ret = notifier.notify(
                message=message,
                title=title,
                retcode=retcode,
                **backend_config)
            if notify_ret:
                ret = notify_ret
        except (SystemExit, KeyboardInterrupt):
            raise
        except Exception as e:
            ret = 1
            if isinstance(e, DefaultNotifierError):
                notifier = e.module
                e = e.exception

            args, _, _, defaults = getargspec(notifier.notify)
            possible_args = set(args)
            required_args =  set(args) if defaults is None else set(args[:-len(defaults)])
            required_args -= set(['title', 'message', 'retcode'])
            unknown_args = set(backend_config) - possible_args
            missing_args = required_args - set(backend_config)

            if unknown_args:
                logging.getLogger(__name__).error(
                    'Got unknown arguments: {}'.format(unknown_args))

            if missing_args:
                logging.getLogger(__name__).error(
                    'Missing arguments: {}'.format(missing_args))

            if not any([unknown_args, missing_args]):
                logging.getLogger(__name__).error(
                    'Failed to send notification using {}'.format(backend),
                    exc_info=True)

    return ret



from os import makedirs, path
from pkgutil import get_data
from sys import argv

from appdirs import user_data_dir

ntfy_data_dir = user_data_dir('ntfy', 'dschep')
if not path.isdir(ntfy_data_dir):
    makedirs(ntfy_data_dir)


class icon(object):
    png = None
    ico = None


progmtime = path.isfile(argv[0]) and path.getmtime(argv[0])
for fmt in ['png', 'ico']:
    icon_path = path.abspath(path.join(ntfy_data_dir, 'icon.' + fmt))
    setattr(icon, fmt, icon_path)
    if not path.isfile(icon_path) or progmtime > path.getmtime(icon_path):
        with open(icon_path, 'wb') as icon_file:
            icon_file.write(get_data('ntfy', 'icon.' + fmt))

scripts = {}
for script in ['auto-ntfy-done.sh', 'bash-preexec.sh']:
    script_path = path.abspath(path.join(ntfy_data_dir, script))
    scripts[script] = script_path
    if not path.isfile(script_path) or progmtime > path.getmtime(script_path):
        with open(script_path, 'wb') as script_file:
            script_file.write(
                get_data('ntfy', path.join('shell_integration', script)))



# -- coding: utf-8 --

import os

from ..data import icon


def notify(title, message, icon=icon.ico, retcode=None):
    """
    Optional parameters:
        * ``icon`` - path to an ICO file to display instead of the ntfy icon
    """

    import win32api
    import win32con
    import win32gui

    class WindowsBalloonTip:
        def __init__(self, title, msg):
            message_map = {
                win32con.WM_DESTROY: self.OnDestroy,
            }
            # Register the Window class.
            wc = win32gui.WNDCLASS()
            hinst = wc.hInstance = win32api.GetModuleHandle(None)
            wc.lpszClassName = "PythonTaskbar"
            wc.lpfnWndProc = message_map  # could also specify a wndproc.
            classAtom = win32gui.RegisterClass(wc)
            # Create the Window.
            style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
            self.hwnd = win32gui.CreateWindow(
                classAtom, "Taskbar", style, 0, 0, win32con.CW_USEDEFAULT,
                win32con.CW_USEDEFAULT, 0, 0, hinst, None)
            win32gui.UpdateWindow(self.hwnd)
            iconPathName = os.path.abspath(icon)
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            try:
                hicon = win32gui.LoadImage(
                    hinst, iconPathName, win32con.IMAGE_ICON, 0, 0, icon_flags)
            except:
                hicon = win32gui.LoadIcon(0, win32con.IDI_APPLICATION)
            flags = win32gui.NIF_ICON | win32gui.NIF_MESSAGE | win32gui.NIF_TIP
            nid = (self.hwnd, 0, flags, win32con.WM_USER + 20, hicon,
                   "tooltip")
            win32gui.Shell_NotifyIcon(win32gui.NIM_ADD, nid)
            win32gui.Shell_NotifyIcon(
                win32gui.NIM_MODIFY,
                (self.hwnd, 0, win32gui.NIF_INFO, win32con.WM_USER + 20, hicon,
                 "Balloon tooltip", title, 200, msg),
            )
            win32gui.DestroyWindow(self.hwnd)
            win32gui.UnregisterClass(wc.lpszClassName, None)

        def OnDestroy(self, hwnd, msg, wparam, lparam):
            win32api.PostQuitMessage(0)  # Terminate the app.

    WindowsBalloonTip(message, title)



import requests

from ..config import USER_AGENT


def notify(title,
           message,
           access_token,
           device_iden=None,
           email=None,
           retcode=None):
    """
    Required parameter:
        * ``access_token`` - Your Pushbullet access token, created at
            https://www.pushbullet.com/#settings/account

    Optional parameters:
        * ``device_iden`` - a device identifier, if omited, notification is
                            sent to all devices
        * ``email`` - send notification to pushbullte user with the specified
                      email or send an email if they aren't a pushullet user
    """

    data = {
        'type': 'note',
        'title': title,
        'body': message,
    }
    if device_iden is not None:
        data['device_iden'] = device_iden
    if email is not None:
        data['email'] = email

    headers = {'Access-Token': access_token, 'User-Agent': USER_AGENT}

    resp = requests.post(
        'https://api.pushbullet.com/v2/pushes', data=data, headers=headers)

    resp.raise_for_status()



from __future__ import print_function

import logging

import requests

from ..config import USER_AGENT


def notify(title,
           message,
           user_key,
           api_token='aUnsraBiEZVsmrG89AZp47K3S2dX2a',
           device=None,
           sound=None,
           priority=0,
           expire=None,
           retry=None,
           callback=None,
           url=None,
           url_title=None,
           html=False,
           retcode=None):
    """
    Required parameters:
        * ``user_key``

    Optional parameters:
        * ``sound``
        * ``priority``
        * ``expire``
        * ``retry``
        * ``callback``
        * ``api_token`` - use your own application token
        * ``device`` - target a device, if omitted, target all devices
        * ``url``
        * ``url_title``
        * ``html``
    """

    data = {
        'message': message,
        'token': api_token,
        'user': user_key,
        'title': title,
    }
    if device:
        data['device'] = device

    if sound:
        data['sound'] = sound

    if url:
        data['url'] = url

    if url_title:
        if not url:
            logging.getLogger(__name__).warning(
                'url_title specified without specifying url')
        else:
            data['url_title'] = url_title

    if html:
        data['html'] = 1

    priority = int(priority)
    if priority <= 2 and priority >= -2:
        if priority != 0:
            data['priority'] = priority

        # Expire, Retry, and Callback only apply to an Emergency Priority
        if priority == 2:
            # Retry can not be less than 30 per the API
            if not retry or retry < 30:
                logging.getLogger(__name__).error(
                    'retry is less than 30 or is not set, '
                    'setting retry to 30 to comply with '
                    'pushover API requirements')
                data['retry'] = 30
            else:
                data['retry'] = retry

            # Expire can not be more than 86400 (24 hours)
            if not expire or expire > 86400:
                logging.getLogger(__name__).error(
                    'expire is greater than 86400 seconds or is not set,'
                    'setting expire to 86400 to comply with'
                    'pushover API requirements')
                data['expire'] = 86400
            elif expire <= 86400:
                data['expire'] = expire

            if callback:
                data['callback'] = callback
        else:
            if retry:
                logging.getLogger(__name__).warning(
                    'Non-emergency, ignoring retry set in config')
            if expire:
                logging.getLogger(__name__).warning(
                    'Non-emergency, ignoring expire set in config')
            if callback:
                logging.getLogger(__name__).warning(
                    'Non-emergency, ignoring callback set in config')

    else:
        raise ValueError('priority must be an integer from -2 to 2')

    resp = requests.post(
        'https://api.pushover.net/1/messages.json',
        data=data,
        headers={
            'User-Agent': USER_AGENT,
        })

    if resp.status_code == 429:
        print("ntfy's default api_token has reached pushover's rate limit")
        print("create your own app at https://pushover.net/apps/clone/ntfy")
        print("and set api_token in your config file.")
        return 1

    resp.raise_for_status()



from rocketchat_API.rocketchat import RocketChat


def notify(title, message, url, username, password, room, retcode=None):

    rocket = RocketChat(username, password, server_url=url)

    msg = "**{}:** {}".format(title, message)
    rocket.chat_post_message(msg, channel=room)



from os import makedirs, path

from appdirs import user_config_dir

from telegram_send import configure, send

config_dir = user_config_dir('ntfy', 'dschep')
config_file = path.join(config_dir, 'telegram.ini')


def notify(title, message, retcode=None):
    """Sends message over Telegram using telegram-send, title is ignored."""
    if not path.exists(config_file):
        if not path.exists(config_dir):
            makedirs(config_dir)
        print("Follow the instructions to configure the Telegram backend.\n")
        configure(config_file)
    send(messages=[message], conf=config_file)



import logging

import requests


def notify(title, message, retcode=None, webhook=None):
    """
    Required parameter:
        * ``webhook`` - The webhook link, created at https://n.tkte.ch/
              (choose ``Plain Text`` service when creating the webhook)
    """

    logger = logging.getLogger(__name__)
    if webhook is None:
        logger.error('please set webhook variable under '
                     'notifico backend of the config file')
        return
    response = requests.get(
        webhook,
        params={
            'payload': '{title}\n{message}'.format(
                title=title, message=message)
        })
    response.raise_for_status()



import requests

from ..config import USER_AGENT


def notify(title,
           message,
           secret,
           endpoint=None,
           level=3,
           link=None,
           retcode=None):
    """
    Required parameter:
        * ``secret`` - The Pushjet service secret token, created with
            http://docs.pushjet.io/docs/creating-a-new-service

    Optional parameters:
        * ``endpoint`` - custom Pushjet API endpoint
            (defaults to https://api.pushjet.io)
        * ``level`` - The importance level from 1(low) to 5(high)
        * ``link``
    """

    data = {
        'title': title,
        'message': message,
        'level': level,
        'secret': secret,
    }

    if link:
        data['link'] = link

    headers = {'User-Agent': USER_AGENT}

    if endpoint is None:
        endpoint = 'https://api.pushjet.io'

    resp = requests.post(endpoint + '/message', data=data, headers=headers)

    resp.raise_for_status()



def notify(title, message, retcode=None):
    """
    adapted from https://gist.github.com/baliw/4020619
    """
    try:
        import Foundation
        import objc
    except ImportError:
        import sys
        import logging

        logger = logging.getLogger(__name__)
        if sys.platform.startswith('darwin') and hasattr(sys, 'real_prefix'):
            logger.error(
                "Using ntfy with the MacOS Notification Center doesn't "
                "work within a virtualenv")
            sys.exit(1)
        else:
            raise

    NSUserNotification = objc.lookUpClass('NSUserNotification')
    NSUserNotificationCenter = objc.lookUpClass('NSUserNotificationCenter')

    notification = NSUserNotification.alloc().init()
    notification.setTitle_(title)
    if message is not None:
        notification.setInformativeText_(message)
    notification.setDeliveryDate_(Foundation.NSDate.date())

    NSUserNotificationCenter.defaultUserNotificationCenter()\
        .scheduleNotification_(notification)



import syslog


def notify(title,
           message,
           prio='ALERT',
           facility='LOCAL5',
           fmt='[{title}] {message}',
           retcode=None):
    """
    Uses the ``syslog`` core Python module, which is not available on Windows
    platforms.

    Optional parameters:
        * ``prio`` - Syslog prority level.  Default is ``ALERT``.  Possible
          values are:

          * EMERG
          * ALERT
          * CRIT
          * ERR
          * WARNING
          * NOTICE
          * INFO
          * DEBUG

        * ``facility`` - Syslog facility.  Default is ``LOCAL5``.  Possible
          values are:

          * KERN
          * USER
          * MAIL
          * DAEMON
          * AUTH
          * LPR
          * NEWS
          * UUCP
          * CRON
          * SYSLOG
          * LOCAL0
          * LOCAL1
          * LOCAL2
          * LOCAL3
          * LOCAL4
          * LOCAL5
          * LOCAL6
          * LOCAL7

      * ``fmt`` - Format of the message to be sent to the system logger.  The
        title and the message are specified using the following placeholders:

        * ``{title}``
        * ``{message}``

        Default is ``[{title}] {message}``.
    """

    prio_map = {
        'EMERG': syslog.LOG_EMERG,
        'ALERT': syslog.LOG_ALERT,
        'CRIT': syslog.LOG_CRIT,
        'ERR': syslog.LOG_ERR,
        'WARNING': syslog.LOG_WARNING,
        'NOTICE': syslog.LOG_NOTICE,
        'INFO': syslog.LOG_INFO,
        'DEBUG': syslog.LOG_DEBUG,
    }

    facility_map = {
        'KERN': syslog.LOG_KERN,
        'USER': syslog.LOG_USER,
        'MAIL': syslog.LOG_MAIL,
        'DAEMON': syslog.LOG_DAEMON,
        'AUTH': syslog.LOG_AUTH,
        'LPR': syslog.LOG_LPR,
        'NEWS': syslog.LOG_NEWS,
        'UUCP': syslog.LOG_UUCP,
        'CRON': syslog.LOG_CRON,
        'SYSLOG': syslog.LOG_SYSLOG,
        'LOCAL0': syslog.LOG_LOCAL0,
        'LOCAL1': syslog.LOG_LOCAL1,
        'LOCAL2': syslog.LOG_LOCAL2,
        'LOCAL3': syslog.LOG_LOCAL3,
        'LOCAL4': syslog.LOG_LOCAL4,
        'LOCAL5': syslog.LOG_LOCAL5,
        'LOCAL6': syslog.LOG_LOCAL6,
        'LOCAL7': syslog.LOG_LOCAL7,
    }

    if prio not in prio_map:
        raise ValueError('invalid syslog priority')
    elif facility not in facility_map:
        raise ValueError('invalid syslog facility')

    msg = fmt.format(title=title, message=message)
    for line in msg.splitlines():
        syslog.syslog(facility_map[facility] | prio_map[prio], line)



import errno
import logging
from os.path import join as join_path
from os.path import expanduser
from sys import exit

import requests
from appdirs import site_config_dir, user_config_dir
from ruamel import yaml

from . import __version__

if yaml.version_info < (0, 15):
    safe_load = yaml.safe_load
else:
    yml = yaml.YAML(typ='safe', pure=True)
    safe_load = lambda stream: yml.load(stream)

DEFAULT_CONFIG = join_path(user_config_dir('ntfy', 'dschep'), 'ntfy.yml')
SITE_DEFAULT_CONFIG = join_path(site_config_dir('ntfy', 'dschep'), 'ntfy.yml')
OLD_DEFAULT_CONFIG = expanduser('~/.ntfy.yml')

USER_AGENT = 'ntfy/{version} {default_user_agent}'.format(
    version=__version__,
    default_user_agent=requests.utils.default_user_agent())


def load_config(config_path=DEFAULT_CONFIG):
    logger = logging.getLogger(__name__)

    try:
        config = safe_load(open(expanduser(config_path)))
    except IOError as e:
        if e.errno == errno.ENOENT and config_path == DEFAULT_CONFIG:
            logger.info('{} not found'.format(config_path))
            config = {}
        else:
            logger.error(
                'Failed to open {}'.format(config_path), exc_info=True)
            exit(1)
    except ValueError as e:
        logger.error('Failed to load {}'.format(config_path), exc_info=True)
        exit(1)

    if 'backend' in config:
        logger.warning(
            "The 'backend' config option is deprecated, use 'backends'")
        if 'backends' in config:
            logger.warning("Both 'backend' and 'backends' in config, "
                           "ignoring 'backend'.")
        else:
            config['backends'] = [config['backend']]

    return config



import logging
import re

from instapush import App


class WrongMessageCountException(Exception):
    pass


class ApiException(Exception):
    pass


def notify(title, message, event_name, appid, secret, trackers, retcode=None):
    """
    Required parameter:
        * ``event_name`` - Instapush event (the notification template)
        * ``appid`` - The appid found on the dashboard
        * ``secret`` - The secret found on the dashboard
        * ``traskers`` - List of the placeholders for the selected event
    """

    logger = logging.getLogger(__name__)
    _msgs = re.split(r'(?<!\\):', message)
    msgs = []

    for msg in _msgs:
        msg = msg.replace("\\:", ":")
        msgs.append(msg)

    if len(msgs) != len(trackers):
        logger.error(('Wrong number of messages! There are {} trackers so you '
                      'have to provide {} messages. Remember to separate each '
                      'message with \':\' (example: send "msg1:msg2")').format(
                          len(trackers), len(trackers)))
        raise WrongMessageCountException()

    to_send = {}

    for tracker, msg in zip(trackers, msgs):
        to_send[tracker] = msg

    app = App(appid=appid, secret=secret)
    res = app.notify(event_name=event_name, trackers=to_send)

    if res["status"] != 200:
        logger.error(res["msg"])
        raise ApiException()



from __future__ import print_function

from subprocess import check_call


def notify(title, message, retcode=None):
    """
    Termux:API backend.
    """

    check_call(['termux-notification', '--content', message, '--title', title])



import requests

from ..config import USER_AGENT

# URL to pushalot.com notification sending endpoint
PUSHALOT_API_URL = 'https://pushalot.com/api/sendmessage'


def notify(title,
           message,
           auth_token,
           source=None,
           url=None,
           url_title=None,
           image=None,
           ttl=None,
           important=False,
           silent=False,
           retcode=None):
    """
    Required parameters:
        * ``auth_token``

    Optional parameters:
        * ``source``
        * ``url``
        * ``url_title``
        * ``image``
        * ``ttl``
        * ``important``
        * ``silent``
    """

    data = {
        'Title': title,
        'Body': message,
        'AuthorizationToken': auth_token,
    }

    if source:
        data['Source'] = source
    if url:
        data['Link'] = url
    if url and url_title:
        data['LinkTitle'] = url_title
    if image:
        data['Image'] = image
    if ttl:
        data['TimeToLive'] = int(ttl)
    if important:
        data['IsImportant'] = 'True'
    if silent:
        data['IsSilent'] = 'True'

    headers = {'User-Agent': USER_AGENT}
    response = requests.post(PUSHALOT_API_URL, data=data, headers=headers)
    response.raise_for_status()



from sys import version_info

from setuptools import find_packages, setup

from ntfy import __version__

deps = ['requests', 'ruamel.yaml', 'appdirs']
extra_deps = {
    ':sys_platform == "win32"': ['pywin32'],
    ':sys_platform == "darwin"': ['pyobjc-core', 'pyobjc'],
    'xmpp': [
        'sleekxmpp', 'dnspython' if version_info[0] < 3 else 'dnspython3'],
    'telegram': ['telegram-send'],
    'instapush': ['instapush'],
    'emoji': ['emoji'],
    'pid':['psutil'],
    'slack':['slacker'],
    'rocketchat':['rocketchat-API'],
    'matrix':['matrix_client'],
}
test_deps = ['mock', 'sleekxmpp', 'emoji', 'psutil']

long_description = open('README.rst').read()

setup(
    name='ntfy',

    version=__version__,

    description='A utility for sending push notifications',
    long_description=long_description,

    url='https://github.com/dschep/ntfy',

    author='Daniel Schep',
    author_email='dschep@gmail.com',

    license='GPLv3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Environment :: Console',

        'Intended Audience :: End Users/Desktop',

        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='push notification',

    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={'ntfy': ['icon.png', 'icon.ico', 'shell_integration/*.sh']},

    install_requires=deps,

    extras_require=extra_deps,

    tests_require=test_deps,
    test_suite='tests',

    entry_points={
        'console_scripts': [
            'ntfy = ntfy.cli:main',
        ],
    },
)



import argparse
import logging
import logging.config
import sys
from os import environ, path
from subprocess import PIPE, STDOUT, Popen
from time import time

from . import __version__, default_title, notify
from .config import (DEFAULT_CONFIG, OLD_DEFAULT_CONFIG, SITE_DEFAULT_CONFIG,
                     load_config)
from .data import scripts

try:
    from shlex import quote as sh_quote
except ImportError:
    from pipes import quote as sh_quote

try:
    from emoji import emojize
except ImportError:
    emojize = None

try:
    import psutil
except ImportError:
    psutil = None

try:
    from .terminal import is_focused
except ImportError:

    def is_focused():
        return True


def run_cmd(args):
    if getattr(args, 'pid', False):
        return watch_pid(args)
    if not args.command:
        if args.formatter:
            args.command, retcode, duration = args.formatter
            args.command, retcode, duration = ([args.command], int(retcode),
                                               int(duration))
            args.option.setdefault('linux', {}).setdefault('transient', 'true')
            stdout, stderr = None, None
        else:
            sys.stderr.write('usage: ntfy done [-h|-L N] command\n'
                             'ntfy done: error: the following arguments '
                             'are required: command\n')
            sys.exit(1)
    else:
        if args.stdout and args.stderr:
            out = PIPE
            err = STDOUT
        else:
            out = PIPE if args.stdout else None
            err = PIPE if args.stderr else None
        start_time = time()
        process = Popen(args.command, stdout=out, stderr=err)
        stdout, stderr = process.communicate()
        process.wait()
        duration = time() - start_time
        retcode = process.returncode
    if args.longer_than is not None and duration <= args.longer_than:
        return None, None
    if args.unfocused_only and is_focused():
        return None, None
    message = _result_message(args.command if not args.hide_command else None,
                              retcode, stdout, stderr, duration,
                              emojize is not None and not args.no_emoji)
    return message, retcode


def _result_message(command, return_code, stdout, stderr, duration, emoji):
    if emoji:
        prefix = ':white_check_mark: ' if return_code == 0 else ':x: '
    else:
        prefix = ''
    if return_code == 0:
        result = 'succeeded'
    else:
        result = 'failed (code {:d})'.format(return_code)
    if command is None:
        command = 'Your command'
    else:
        command = '"{command}"'.format(command=' '.join(command))
    if stdout is not None or stderr is not None:
        all_output = ':\n{}{}'.format(stdout
                                      if stdout is not None else '', stderr
                                      if stderr is not None else '')
    else:
        all_output = ''
    template = '{prefix}{command} {result} in {:d}:{:02d} minutes{output}'
    return template.format(
        prefix=prefix,
        command=command,
        result=result,
        output=all_output,
        *map(int, divmod(duration, 60)))


def watch_pid(args):
    if psutil is None:  # pragma: no cover
        logging.error(
            "This command requires psutil module. Pleases install psutil.")
        sys.exit(1)
    try:
        p = psutil.Process(args.pid)
        cmd = p.cmdline()
        start_time = p.create_time()
    except psutil.NoSuchProcess:
        logging.error("PID {} not found".format(args.pid))
        sys.exit(1)
    ret = None
    try:
        ret = p.wait()
    except psutil.NoSuchProcess:  # pragma: no cover
        pass  # this happens when the PID disapears
    duration = time() - start_time
    return 'PID[{}]: "{}" finished in {:d}:{:02d} minutes'.format(
        p.pid, ' '.join(cmd), *map(int, divmod(duration, 60))), ret


def auto_done(args):
    if args.longer_than:
        print('export AUTO_NTFY_DONE_LONGER_THAN=-L{}'.format(
            args.longer_than))
    if args.unfocused_only:
        print('export AUTO_NTFY_DONE_UNFOCUSED_ONLY=-b')
    if args.shell == 'bash':
        print('source {}'.format(sh_quote(scripts['bash-preexec.sh'])))
    print('source {}'.format(sh_quote(scripts['auto-ntfy-done.sh'])))
    print("# To use ntfy's shell integration, run "
          "this and add it to your shell's rc file:")
    print('# eval "$(ntfy shell-integration)"')
    return None, None


class BackendOptionAction(argparse.Action):
    backend = None

    def __call__(self, parser, args, values, option_string=None):
        if self.dest == 'backend':
            self.__class__.backend = values
            if args.backend is None:
                args.backend = []
            args.backend.append(values)
        elif self.dest == 'option':
            if args.option is None:
                args.option = {}
            args.option.setdefault(self.__class__.backend,
                                   {})[values[0]] = values[1]
        else:
            raise Exception("'BackendOptionAction only supports dest of "
                            "'backend' and 'option'")


parser = argparse.ArgumentParser(
    description='Send push notification when command finishes')

parser.add_argument(
    '-c',
    '--config',
    help='config file to use (default: {})'.format(DEFAULT_CONFIG))
parser.add_argument(
    '-b',
    '--backend',
    action=BackendOptionAction,
    help='override backend specified in config')
parser.add_argument(
    '-o',
    '--option',
    nargs=2,
    default=None,
    action=BackendOptionAction,
    metavar=('key', 'value'),
    help='backend specific options')
parser.add_argument(
    '-l',
    '--log-level',
    action='store',
    default='WARNING',
    choices=['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'],
    help=('Specify the how verbose CLI output is '
          '(default: WARNING)'))
parser.add_argument(
    '-v',
    '--verbose',
    dest='log_level',
    action='store_const',
    const='DEBUG',
    help='a shortcut for --log-level=DEBUG')
parser.add_argument(
    '-q',
    '--quiet',
    dest='log_level',
    action='store_const',
    const='CRITICAL',
    help='a shortcut for --log-level=CRITICAL')
parser.add_argument('--version', action='version', version=__version__)
if emojize is not None:
    parser.add_argument(
        '-E', '--no-emoji', action='store_true', help='Disable emoji support')

parser.add_argument(
    '-t',
    '--title',
    help='a title for the notification (default: {})'.format(default_title))

subparsers = parser.add_subparsers()

send_parser = subparsers.add_parser('send', help='send a notification')
send_parser.add_argument('message', help='notification message')


def default_sender(args):
    return args.message, 0


send_parser.set_defaults(func=default_sender)

done_parser = subparsers.add_parser(
    'done', help='run a command and send a notification when done')
done_parser.add_argument(
    'command', nargs=argparse.REMAINDER, help='command to run')
done_parser.add_argument(
    '-L',
    '--longer-than',
    type=int,
    metavar='N',
    help="Only notify if the command runs longer than N seconds")
done_parser.add_argument(
    '-b',
    '--background-only',
    action='store_true',
    default=False,
    dest='unfocused_only',
    help="Only notify if shell isn't in the foreground")
done_parser.add_argument(
    '--formatter',
    metavar=('command', 'retcode', 'duration'),
    nargs=3,
    help="Format and send cmd, retcode & duration instead of running command. "
    "Used internally by shell-integration")
if psutil is not None:
    done_parser.add_argument(
        '-p',
        '--pid',
        type=int,
        help="Watch a PID instead of running a new command")
done_parser.add_argument(
    '-o',
    '--stdout',
    action='store_true',
    help="Capture and send standard output")
done_parser.add_argument(
    '-e',
    '--stderr',
    action='store_true',
    help="Capture and send standard error")
done_parser.add_argument(
    '-H',
    '--hide-command',
    action='store_true',
    default=False,
    help="Do not display the executed command in any notifications")
done_parser.set_defaults(func=run_cmd)

shell_integration_parser = subparsers.add_parser(
    'shell-integration',
    help='automatically get notifications when long running commands finish')
shell_integration_parser.add_argument(
    '-s',
    '--shell',
    default=path.split(environ.get('SHELL', ''))[1],
    choices=['bash', 'zsh'],
    help='The shell to integrate ntfy with (default: $SHELL)')
shell_integration_parser.add_argument(
    '-L',
    '--longer-than',
    default=10,
    type=int,
    metavar='N',
    help="Only notify if the command runs longer than N seconds")
shell_integration_parser.add_argument(
    '-f',
    '--foreground-too',
    action='store_false',
    default=True,
    dest='unfocused_only',
    help="Also notify if shell is in the foreground")
shell_integration_parser.set_defaults(func=auto_done)


def main(cli_args=None):
    if cli_args is not None:
        args = parser.parse_args(cli_args)
    else:
        args = parser.parse_args()

    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'default': {
                'format': '%(levelname)s: %(message)s'
            },
        },
        'handlers': {
            'default': {
                'level': args.log_level,
                'class': 'logging.StreamHandler',
                'formatter': 'default',
            },
        },
        'loggers': {
            '': {
                'handlers': ['default'],
                'level': args.log_level,
                'propagate': True,
            }
        }
    })

    if args.config is not None:
        config = load_config(args.config)
    elif path.exists(path.expanduser(DEFAULT_CONFIG)):
        config = load_config(DEFAULT_CONFIG)
    elif path.exists(OLD_DEFAULT_CONFIG):
        config = load_config(OLD_DEFAULT_CONFIG)
    elif path.exists(path.expanduser(SITE_DEFAULT_CONFIG)):
        config = load_config(SITE_DEFAULT_CONFIG)
    else:  # get default config and print message about missing file
        config = load_config()

    if 'NTFY_BACKENDS' in environ:
        config['backends'] = environ['NTFY_BACKENDS'].split(',')

    if args.backend:
        config['backends'] = args.backend

    if args.option is None:
        args.option = {}
    for backend, backend_options in args.option.items():
        if backend is not None:
            config.setdefault(backend, {}).update(backend_options)

    if getattr(args, 'func', None) == run_cmd and args.longer_than is None and\
            'longer_than' in config:
        args.longer_than = config['longer_than']

    if getattr(args, 'func', None) == run_cmd and 'hide_command' in config:
        args.hide_command = config['hide_command']

    if hasattr(args, 'func'):
        message, retcode = args.func(args)
        if message is None:
            return 0
        if emojize is not None and not args.no_emoji:
            message = emojize(message, use_aliases=True)
        return notify(
            message,
            args.title,
            config,
            retcode=retcode,
            **dict(args.option.get(None, [])))
    else:
        parser.print_help()


if __name__ == '__main__':
    sys.exit(main())
